############################# SMiP 2024 MLM Example 1  #########################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

# To import xls or xlsx data as table
if (!require("readxl")) install.packages("readxl"); library(readxl)

# To get compact data description
if (!require("psych")) install.packages("psych"); library(psych) 

# To estimate MLMs using gls or lme
if (!require("nlme")) install.packages("nlme"); library(nlme) 

# To estimate MLMs using lmer
# Re-install to fix problem of matrix incompatibility
oo <- options(repos = "https://cran.r-project.org/")
utils::install.packages("Matrix")
utils::install.packages("lme4")
options(oo)
library(Matrix); library(lme4)

# To get Satterthwaite DDF in lmer
if (!require("lmerTest")) install.packages("lmerTest"); library(lmerTest) 

# To get ICC in lmer
if (!require("performance")) install.packages("performance"); library(performance)

# Clear environment (re-run as needed for troubleshooting purposes)
rm(list=ls())


################################################################################
#####       DATA IMPORT, MANIPULATION, AND PREPARATION FOR EXAMPLE 1       #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/Papers and Data/SMiP/SMiP_2024_MLM_Part1/"
filename = "Example1.xlsx"
setwd(dir=filesave)

# Import Example 1 excel trial-level dataset
Example1 = read_excel(paste0(filesave,filename)) 
# Convert to data frame to use in analysis
Example1 = as.data.frame(Example1)

# Filter to only cases complete on all variables to be used below
Example1 = Example1[complete.cases(Example1[ , c("RT","freq01","size01")]),]

print("Descriptives for RT across all trials")
describe(x=Example1[,c("RT")], fast=TRUE)
print("Variance of RT across all trials")
var(x=Example1[,c("RT")])

# Add mean subject RT and number of responses per subject to dataset
Example1$SubjectMeanRT = ave(x=Example1$RT, Example1$SubjectID, FUN=mean)
Example1$SubjectN = ave(x=Example1$RT, Example1$SubjectID, FUN=length)
# Create subject-level dataset
Example1Subjects = unique(Example1[,c("SubjectID","SubjectMeanRT","SubjectN")])
print("Descriptives for SubjectMeanRT and SubjectN across subjects")
describe(x=Example1Subjects[,c("SubjectMeanRT","SubjectN")], fast=TRUE)

# Add mean item RT and number of responses per item to dataset
Example1$ItemMeanRT = ave(x=Example1$RT, Example1$ItemID, FUN=mean)
Example1$ItemN = ave(x=Example1$RT, Example1$ItemID, FUN=length)
# Create item-level dataset to see how many items there are of each kind
Example1Items = unique(Example1[,c("ItemID","freq01","size01","ItemMeanRT","ItemN")])
print("Cross-tabulation of item predictors")
table(Example1Items$freq01,Example1Items$size01)
print("Descriptives for ItemMeanRT and ItemN across items")
describe(x=Example1Items[,c("ItemMeanRT","ItemN")], fast=TRUE)


################################################################################
#####             BEGIN EMPTY MEANS MODELS FOR EXAMPLE 1                   #####
################################################################################

print("Empty Model 1: Single-level ignoring dependency of subjects and items")
print("Using gls instead of lm to get model log-likelihood for model comparison")
Empty1 = gls(data=Example1, method="REML", model=RT~1)
print("Show default results"); summary(Empty1) 
print("Model -2LL = "); -2*logLik(Empty1) 
print("Level-1 residual variance = "); summary(Empty1)$sigma^2 

################################################################################
print("Empty Model 2: Two-level nested for trials nested in subjects only")
Empty2 = lmer(data=Example1, REML=TRUE, formula=RT~1+(1|SubjectID))
print("Show results using Sattherwaite DDF") 
summary(Empty2, ddf="Satterthwaite")

print("Show intraclass correlation"); icc(Empty2)

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Empty2))
# Compute model-implied total variance
Empty2TotVar = as.data.frame(VarCorr(Empty2))[1,4] +
               as.data.frame(VarCorr(Empty2))[2,4] 
print("Total model-implied variance = "); Empty2TotVar
# Save subject random intercept variance as object to show computation of ICC
Empty2SubVar = as.data.frame(VarCorr(Empty2))[1,4]
# Manual computation of subject ICC
print("Subject ICC = "); Empty2ICC_S = Empty2SubVar/Empty2TotVar; Empty2ICC_S

print("Show intraclass correlation LRT"); ranova(Empty2)

################################################################################
print("Empty Model 3: Two-level crossed for trials nested in subjects and in items")
Empty3 = lmer(data=Example1, REML=TRUE, RT~1+(1|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Empty3, ddf="Satterthwaite")

print("Show intraclass correlations for proportion of variance due to each sampling dimension") 
icc(Empty3, by_group=TRUE)

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Empty3))
# Compute model-implied total variance
Empty3TotVar = as.data.frame(VarCorr(Empty3))[1,4] +
               as.data.frame(VarCorr(Empty3))[2,4] +
               as.data.frame(VarCorr(Empty3))[3,4]
print("Total model-implied variance = "); Empty3TotVar
# Save each variance as an object
Empty3ItemIntVar = as.data.frame(VarCorr(Empty3))[1,4]
Empty3SubIntVar  = as.data.frame(VarCorr(Empty3))[2,4]
Empty3ResVar     = as.data.frame(VarCorr(Empty3))[3,4]
# Manual computation of subject and item ICCs
print("Subject ICC = "); Empty3ICC_S = Empty3SubIntVar /Empty3TotVar; Empty3ICC_S
print("Item ICC = ");    Empty3ICC_I = Empty3ItemIntVar/Empty3TotVar; Empty3ICC_I

print("Show intraclass correlation LRTs"); ranova(Empty3)

# Save fixed intercept for use below
Empty3FixInt = fixef(Empty3)
print("95% random intercept confidence interval for subjects")
SubInt_LCI = Empty3FixInt - 1.96*sqrt(Empty3SubIntVar)
SubInt_UCI = Empty3FixInt + 1.96*sqrt(Empty3SubIntVar) 
print("Subject intercept lower CI = "); SubInt_LCI
print("Subject intercept upper CI = "); SubInt_UCI
print("95% random intercept confidence interval for items")
ItemInt_LCI = Empty3FixInt - 1.96*sqrt(Empty3ItemIntVar) 
ItemInt_UCI = Empty3FixInt + 1.96*sqrt(Empty3ItemIntVar)
print("Item intercept lower CI = "); ItemInt_LCI
print("Item intercept upper CI = "); ItemInt_UCI


################################################################################
#####                 BEGIN CONDITIONAL MODELS FOR EXAMPLE 1               #####
################################################################################

print("Conditional Model 1: Add fixed slopes for item predictors")
Cond1 = lmer(data=Example1, REML=TRUE, formula=
             RT~1+freq01+size01+freq01:size01+(1|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond1, ddf="Satterthwaite")

# Btw, here is shorter code to include interactions with all lower-order main effects:
#Cond1 = lmer(data=Example1, REML=TRUE, 
#             RT~1+freq01*size01+(1|SubjectID)+(1|ItemID))

# Cell means (the manual way)
print("RT for low  freq, small size"); contest1D(Cond1, L=c(1,0,0,0))
print("RT for low  freq, large size"); contest1D(Cond1, L=c(1,0,1,0))
print("RT for high freq, small size"); contest1D(Cond1, L=c(1,1,0,0))
print("RT for high freq, large size"); contest1D(Cond1, L=c(1,1,1,1))
  
# Simple effects of freq per size
print("Freq slope for small size"); contest1D(Cond1, L=c(0,1,0,0))
print("Freq slope for large size"); contest1D(Cond1, L=c(0,1,0,1))
  
# Simple effects of size per freq
print("Size slope for low freq");  contest1D(Cond1, L=c(0,0,1,0))
print("Size slope for high freq"); contest1D(Cond1, L=c(0,0,1,1))
  
# Interaction (repeated for convenience, two possible interpretations)
print("Size slope diff for low vs high freq");    contest1D(Cond1, L=c(0,0,0,1))
print("Freq slope diff for small vs large size"); contest1D(Cond1, L=c(0,0,0,1))
  
# Make a plot of predicted outcomes
# Create columns of values to be changed across fake items (FI)
FakeFreq = c(0,1,0,1)
FakeSize = c(0,0,1,1)
# Create dataset using just-created columns and constants for other model variables
FakeItems = data.frame(SubjectID=-99, ItemID=-99, freq01=FakeFreq, size01=FakeSize)

# Merge predicted values from main-effects-only model into FI data
FakeItems = data.frame(FakeItems, yhat=predict(object=Cond1, newdata=FakeItems, re.form=NA))

# Make and save plot
png(file = "Example1_Plot.png")  # open file
plot(y=FakeItems$yhat, x=FakeItems$freq01, type="n", ylim=c(500,700), xlim=c(0,1),
     xlab="Frequency (0=low, 1=high)", ylab="Predicted RT")
lines(x=FakeItems$freq01[1:2], y=FakeItems$yhat[1:2], type="l", col="blue1")
lines(x=FakeItems$freq01[3:4], y=FakeItems$yhat[3:4], type="l", col="red1")
legend(x=.05, y=690, legend=c("Size 0=small","Size 1=large"), 
       col=c("blue1","red1"), lty=1) #lty=linetype
dev.off()  # close file

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Cond1))
# Save each variance as an object
Cond1ItemIntVar = as.data.frame(VarCorr(Cond1))[1,4]
Cond1SubIntVar  = as.data.frame(VarCorr(Cond1))[2,4]
Cond1ResVar     = as.data.frame(VarCorr(Cond1))[3,4]
# Compute pseudo-R2 for each variance
Cond1ItemIntR2 = (Empty3ItemIntVar-Cond1ItemIntVar)/Empty3ItemIntVar; Cond1ItemIntR2
Cond1SubIntR2  = (Empty3SubIntVar- Cond1SubIntVar) /Empty3SubIntVar;  Cond1SubIntR2
Cond1ResR2     = (Empty3ResVar-    Cond1ResVar)    /Empty3ResVar;     Cond1ResR2
  
print("Omnibus F-test for model prediction")
contestMD(Cond1, ddf="Satterthwaite", L=rbind(c(0,1,0,0),c(0,0,1,0),c(0,0,0,1)))

print("LRT for remaining item random intercept variance"); ranova(Cond1)

#################################################################################
print("Conditional Model 2: Add random slope across subjects for item freq")
Cond2 = lmer(data=Example1, REML=TRUE, formula=
             RT~1+freq01+size01+freq01:size01+(1+freq01|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond2, ddf="Satterthwaite")
print("LRT for subject random slope of item freq"); ranova(Cond2)

print("LRT for random slope variance using a mixture-chi-square test")
Cond2Diff2LL = -2*(logLik(Cond1)-logLik(Cond2)); Cond2Diff2LL
Cond2DiffP1 = pchisq(Cond2Diff2LL, df=1, lower.tail=FALSE); Cond2DiffP1 
Cond2DiffP2 = pchisq(Cond2Diff2LL, df=2, lower.tail=FALSE); Cond2DiffP2
Cond2DiffP12 = (.5*Cond2DiffP1) + (.5*Cond2DiffP2)
print("Test statistic and mixture p-values for df=1,2") 
Cond2Diff2LL; Cond2DiffP12

# Print stored fixed effects table for reference
as.data.frame(fixef(Cond2))
# Save fixed freq01 slope for use below
Cond2FreqSlp = as.data.frame(fixef(Cond2))[2,1]
# Print stored variance components table for reference
as.data.frame(VarCorr(Cond2))
# Save subject random freq01 slope variance for use below
Cond2SubFreqVar = as.data.frame(VarCorr(Cond2))[3,4]      
print("95% random freq01 slope confidence interval for subjects")
SubFreqSlp_LCI = Cond2FreqSlp - 1.96*sqrt(Cond2SubFreqVar) 
SubFreqSlp_UCI = Cond2FreqSlp + 1.96*sqrt(Cond2SubFreqVar)
print("Subject freq01 slope lower CI = "); SubFreqSlp_LCI
print("Subject freq01 slope upper CI = "); SubFreqSlp_UCI

#################################################################################
print("Conditional Model 3: Add random slope across subjects for item size")
Cond3 = lmer(data=Example1, REML=TRUE, formula=
             RT~1+freq01+size01+freq01:size01+(1+size01|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond3, ddf="Satterthwaite")
print("LRT for subject random slope of item freq"); ranova(Cond3)

print("LRT for random slope variance using a mixture-chi-square test")
Cond3Diff2LL = -2*(logLik(Cond1)-logLik(Cond3)); Cond3Diff2LL
Cond3DiffP1 = pchisq(Cond3Diff2LL, df=1, lower.tail=FALSE); Cond3DiffP1 
Cond3DiffP2 = pchisq(Cond3Diff2LL, df=2, lower.tail=FALSE); Cond3DiffP2
Cond3DiffP12 = (.5*Cond3DiffP1) + (.5*Cond3DiffP2)
print("Test statistic and mixture p-values for df=1,2") 
Cond3Diff2LL; Cond3DiffP12

# Print stored fixed effects table for reference
as.data.frame(fixef(Cond3))
# Save fixed freq01 slope for use below
Cond3SizeSlp = as.data.frame(fixef(Cond3))[3,1]
# Print stored variance components table for reference
as.data.frame(VarCorr(Cond3))
# Save subject random size01 slope variance for use below
Cond3SubSizeVar = as.data.frame(VarCorr(Cond3))[3,4]      
print("95% random size01 slope confidence interval for subjects")
SubSizeSlp_LCI = Cond3SizeSlp - 1.96*sqrt(Cond3SubSizeVar)
SubSizeSlp_UCI = Cond3SizeSlp + 1.96*sqrt(Cond3SubSizeVar) 
print("Subject size01 slope lower CI = "); SubSizeSlp_LCI
print("Subject size01 slope upper CI = "); SubSizeSlp_UCI

