############################# SMiP 2024 MLM Example 2  #########################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=5, scipen=6, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

# To import xls or xlsx data as table
if (!require("readxl")) install.packages("readxl"); library(readxl)

# To get compact data description
if (!require("psych")) install.packages("psych"); library(psych) 

# To estimate MLMs using gls or lme
if (!require("nlme")) install.packages("nlme"); library(nlme) 

# To estimate MLMs using lmer
# Re-install to fix problem of matrix incompatibility
#oo <- options(repos = "https://cran.r-project.org/")
#utils::install.packages("Matrix")
#utils::install.packages("lme4")
#options(oo)
library(Matrix); library(lme4)

# To get Satterthwaite DDF in lmer
if (!require("lmerTest")) install.packages("lmerTest"); library(lmerTest) 

# To get ICC in lmer
if (!require("performance")) install.packages("performance"); library(performance)

# Clear environment (re-run as needed for troubleshooting purposes)
rm(list=ls())


################################################################################
#####       DATA IMPORT, MANIPULATION, AND PREPARATION FOR EXAMPLE 2       #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/Papers and Data/SMiP/SMiP_2024_MLM_Part1/"
filename = "Example2.xlsx"; sheetname = "data"
setwd(dir=filesave)

# Import Example 2 excel trial-level dataset
Example2 = read_excel(paste0(filesave,filename), sheet=sheetname) 
# Convert to data frame to use in analysis
Example2 = as.data.frame(Example2)

# Create predictors for analysis
# Log-transformed RT
Example2$logRT = log(Example2$RT) 
# Piecewise slope for years over 65 if in older group
Example2$yrs65=NA
Example2$yrs65[which(Example2$older==0)]=0
Example2$yrs65[which(Example2$older==1)]=Example2$age[which(Example2$older==1)]-65
# Center item predictors near middle of 0-5 scale
Example2$rel3 = Example2$relevance-3
Example2$sal3 = Example2$salience-3

# Filter to only cases complete on all variables to be used below
Example2 = Example2[complete.cases(Example2[ , c("logRT","rel3","sal3","older","yrs65")]),]

print("Descriptives for logRT and predictors across all trials")
print(describe(x=Example2[,c("logRT","relevance","salience","age")], fast=TRUE), digits=3)

# Add mean subject RT and number of responses per subject to dataset
Example2$SubjectMeanRT = ave(x=Example2$logRT, Example2$SubjectID, FUN=mean)
Example2$SubjectN = ave(x=Example2$logRT, Example2$SubjectID, FUN=length)
# Create subject-level dataset
Example2Subjects = unique(Example2[,c("SubjectID","SubjectMeanRT","SubjectN")])
print("Descriptives for SubjectMeanRT and SubjectN across subjects")
print(describe(x=Example2Subjects[,c("SubjectMeanRT","SubjectN")], fast=TRUE), digits=3)

# Add mean item RT and number of responses per item to dataset
Example2$ItemMeanRT = ave(x=Example2$logRT, Example2$ItemID, FUN=mean)
Example2$ItemN = ave(x=Example2$logRT, Example2$ItemID, FUN=length)
# Create item-level dataset
Example2Items = unique(Example2[,c("ItemID","ItemMeanRT","ItemN")])
print("Descriptives for ItemMeanRT and ItemN across items")
print(describe(x=Example2Items[,c("ItemMeanRT","ItemN")], fast=TRUE), digits=3)


################################################################################
#####             BEGIN EMPTY MEANS MODELS FOR EXAMPLE 2                   #####
################################################################################

# Skipping empty models 1 and 2 (were only for pedagogical purposes)

print("Empty Model 3: Two-level crossed for trials nested in subjects and in items")
Empty3 = lmer(data=Example2, REML=TRUE, formula=logRT~1+(1|SubjectID)+(1|ItemID))
print("Show results using Satttherwaite DDF") 
summary(Empty3, ddf="Satterthwaite")

print("Show intraclass correlations for proportion of variance due to each sampling dimension") 
icc(Empty3, by_group=TRUE)

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Empty3))
# Compute model-implied total variance
Empty3TotVar = as.data.frame(VarCorr(Empty3))[1,4] +
               as.data.frame(VarCorr(Empty3))[2,4] +
               as.data.frame(VarCorr(Empty3))[3,4]
print("Total model-implied variance = "); Empty3TotVar
# Save each variance as an object
Empty3SubIntVar  = as.data.frame(VarCorr(Empty3))[1,4]
Empty3ItemIntVar = as.data.frame(VarCorr(Empty3))[2,4]
Empty3ResVar     = as.data.frame(VarCorr(Empty3))[3,4]
# Manual computation of subject and item ICCs
print("Subject ICC = "); Empty3ICC_S = Empty3SubIntVar /Empty3TotVar; Empty3ICC_S
print("Item ICC = ");    Empty3ICC_I = Empty3ItemIntVar/Empty3TotVar; Empty3ICC_I

print("Show intraclass correlation LRTs"); ranova(Empty3)

# Save fixed intercept for use below
Empty3FixInt = fixef(Empty3)
print("95% random intercept confidence interval for subjects")
SubInt_LCI = Empty3FixInt - 1.96*sqrt(Empty3SubIntVar)
SubInt_UCI = Empty3FixInt + 1.96*sqrt(Empty3SubIntVar) 
print("Subject intercept lower CI = "); SubInt_LCI
print("Subject intercept upper CI = "); SubInt_UCI
print("95% random intercept confidence interval for items")
ItemInt_LCI = Empty3FixInt - 1.96*sqrt(Empty3ItemIntVar) 
ItemInt_UCI = Empty3FixInt + 1.96*sqrt(Empty3ItemIntVar)
print("Item intercept lower CI = "); ItemInt_LCI
print("Item intercept upper CI = "); ItemInt_UCI


################################################################################
#####                 BEGIN CONDITIONAL MODELS FOR EXAMPLE 2               #####
################################################################################

print("Conditional Model 1: Add fixed slopes for item predictors")
Cond1 = lmer(data=Example2, REML=TRUE, formula=
             logRT~1+rel3+sal3+rel3:sal3+(1|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond1, ddf="Satterthwaite")

# Btw, here is shorter code to include interactions with all lower-order main effects:
#Cond1 = lmer(data=Example1, REML=TRUE, 
#             logRT~1+rel3*sal3+(1|SubjectID)+(1|ItemID))

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Cond1))
# Save each variance as an object
Cond1SubIntVar  = as.data.frame(VarCorr(Cond1))[1,4]
Cond1ItemIntVar = as.data.frame(VarCorr(Cond1))[2,4]
Cond1ResVar     = as.data.frame(VarCorr(Cond1))[3,4]
# Compute pseudo-R2 for each variance
Cond1SubIntR2  = (Empty3SubIntVar  - Cond1SubIntVar) /Empty3SubIntVar;  Cond1SubIntR2
Cond1ItemIntR2 = (Empty3ItemIntVar - Cond1ItemIntVar)/Empty3ItemIntVar; Cond1ItemIntR2
Cond1ResR2     = (Empty3ResVar     - Cond1ResVar)    /Empty3ResVar;     Cond1ResR2

print("Omnibus F-test for item fixed slopes")
contestMD(Cond1, ddf="Satterthwaite", L=rbind(c(0,1,0,0),c(0,0,1,0),c(0,0,0,1)))

print("LRT for remaining item random intercept variance"); ranova(Cond1)

################################################################################
print("Conditional Model 2: Add fixed slopes for subject predictors")
Cond2 = lmer(data=Example2, REML=TRUE, formula= 
             logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
                  (1|SubjectID)+(1|ItemID)) # Moved interaction to end to match output
print("Show results using Sattherwaite DDF") 
summary(Cond2, ddf="Satterthwaite")

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Cond2))
# Save each variance as an object
Cond2SubIntVar  = as.data.frame(VarCorr(Cond2))[1,4]
Cond2ItemIntVar = as.data.frame(VarCorr(Cond2))[2,4]
Cond2ResVar     = as.data.frame(VarCorr(Cond2))[3,4]
# Compute pseudo-R2 for each variance and change relative to Cond1
Cond2SubIntR2        = (Empty3SubIntVar  - Cond2SubIntVar) /Empty3SubIntVar;  Cond2SubIntR2
Cond2ItemIntR2       = (Empty3ItemIntVar - Cond2ItemIntVar)/Empty3ItemIntVar; Cond2ItemIntR2
Cond2ResR2           = (Empty3ResVar     - Cond2ResVar)    /Empty3ResVar;     Cond2ResR2
Cond2SubIntR2Change  =  Cond2SubIntR2  - Cond1SubIntR2;  Cond2SubIntR2Change
Cond2ItemIntR2Change =  Cond2ItemIntR2 - Cond1ItemIntR2; Cond2ItemIntR2Change
Cond2ResR2Change     =  Cond2ResR2     - Cond1ResR2;     Cond2ResR2Change

print("Omnibus F-test for subject fixed slopes")
contestMD(Cond2, ddf="Satterthwaite", L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0)))

print("LRT for remaining subject random intercept variance"); ranova(Cond2)

################################################################################
print("Conditional Model 3: Add random subject slope for item predictor sal3")
Cond3 = lmer(data=Example2, REML=TRUE, formula= 
             logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
                  (1+sal3|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond3, ddf="Satterthwaite")
print("LRT for subject item salience slope variance"); ranova(Cond3)

print("LRT for random slope variance using a mixture-chi-square test")
Cond3Diff2LL = -2*(logLik(Cond2)-logLik(Cond3)); Cond3Diff2LL
Cond3DiffP1 = pchisq(Cond3Diff2LL, df=1, lower.tail=FALSE); Cond3DiffP1 
Cond3DiffP2 = pchisq(Cond3Diff2LL, df=2, lower.tail=FALSE); Cond3DiffP2
Cond3DiffP12 = (.5*Cond3DiffP1) + (.5*Cond3DiffP2)
print("Test statistic and mixture p-values for df=1,2") 
Cond3Diff2LL; Cond3DiffP12

# Print stored fixed effects table for reference
as.data.frame(fixef(Cond3))
# Save fixed sal3 slope for use below
Cond3SalSlp = as.data.frame(fixef(Cond3))[3,1]

# Print stored variance components table for reference
as.data.frame(VarCorr(Cond3))
# Save each variance as an object
Cond3SubIntVar  = as.data.frame(VarCorr(Cond3))[1,4]
Cond3SubSalVar  = as.data.frame(VarCorr(Cond3))[2,4]
Cond3ItemIntVar = as.data.frame(VarCorr(Cond3))[4,4]
Cond3ResVar     = as.data.frame(VarCorr(Cond3))[5,4]

print("95% random salience slope confidence interval for subjects")
SubSal_LCI = Cond3SalSlp - 1.96*sqrt(Cond3SubSalVar) 
SubSal_UCI = Cond3SalSlp + 1.96*sqrt(Cond3SubSalVar)
print("Subject sal3 slope lower CI = "); SubSal_LCI
print("Subject sal3 slope upper CI = "); SubSal_UCI

################################################################################
print("Conditional Model 4: Add random subject slope for item predictor rel3")
Cond4 = lmer(data=Example2, REML=TRUE, formula= 
             logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
                  (1+sal3+rel3|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond4, ddf="Satterthwaite")
print("LRT for subject item salience slope variance"); ranova(Cond4)

print("LRT for random slope variance using a mixture-chi-square test")
Cond4Diff2LL = -2*(logLik(Cond3)-logLik(Cond4)); Cond4Diff2LL
Cond4DiffP2 = pchisq(Cond4Diff2LL, df=2, lower.tail=FALSE); Cond4DiffP2 
Cond4DiffP3 = pchisq(Cond4Diff2LL, df=3, lower.tail=FALSE); Cond4DiffP3
Cond4DiffP23 = (.5*Cond4DiffP2) + (.5*Cond4DiffP3)
print("Test statistic and mixture p-values for df=2,3") 
Cond4Diff2LL; Cond4DiffP23

# Print stored fixed effects table for reference
as.data.frame(fixef(Cond4))
# Save fixed rel3 slope for use below
Cond4RelSlp = as.data.frame(fixef(Cond4))[2,1]

# Print stored variance components table for reference
as.data.frame(VarCorr(Cond4))
# Save subject random rel3 slope variance for use below
Cond4SubRelVar = as.data.frame(VarCorr(Cond4))[3,4] 

print("95% random relevance slope confidence interval for subjects")
SubRelSlp_LCI = Cond4RelSlp - 1.96*sqrt(Cond4SubRelVar) 
SubRelSlp_UCI = Cond4RelSlp + 1.96*sqrt(Cond4SubRelVar)
print("Subject rel3 slope lower CI = "); SubRelSlp_LCI
print("Subject rel3 slope upper CI = "); SubRelSlp_UCI

################################################################################
print("Conditional Model 5: Add all possible fixed slope interactions")
Cond5 = lmer(data=Example2, REML=TRUE, formula= 
             logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
                  rel3:older+ sal3:older+ rel3:sal3:older+
                  rel3:yrs65+ sal3:yrs65+ rel3:sal3:yrs65+
                  (1+sal3|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond5, ddf="Satterthwaite")

# Btw, here is shorter code to include interactions with all lower-order main effects:
#Cond5 = lmer(data=Example2, REML=TRUE, formula= 
#             logRT~1+rel3*sal3*older + rel3*sal3*yrs65+
#             (1+sal3|SubjectID)+(1|ItemID))

print("Omnibus F-test for fixed slopes for interactions with older")
contestMD(Cond5, ddf="Satterthwaite", L=rbind(c(0,0,0,0,0,0,1,0,0,0,0,0),
                 c(0,0,0,0,0,0,0,1,0,0,0,0),c(0,0,0,0,0,0,0,0,0,0,1,0)))

print("Omnibus F-test for fixed slopes for interactions with yrs65")
contestMD(Cond5, ddf="Satterthwaite", L=rbind(c(0,0,0,0,0,0,0,0,1,0,0,0),
                 c(0,0,0,0,0,0,0,0,0,1,0,0),c(0,0,0,0,0,0,0,0,0,0,0,1)))

################################################################################
print("Conditional Model 6: Remove 3 nonsignificant yrs65 interactions")
Cond6 = lmer(data=Example2, REML=TRUE, formula=  
             logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
                  rel3:older+ sal3:older+ rel3:sal3:older+
                  (1+sal3|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond6, ddf="Satterthwaite")

# Get simple slopes to understand pattern of three-way interaction
print("Rel slope at sal=3 for younger"); contest1D(Cond6, L=c(0,1,0,0,0,0,0,0,0))
print("Rel slope at sal=3 for older");   contest1D(Cond6, L=c(0,1,0,0,0,0,1,0,0))
print("Sal slope at rel=3 for younger"); contest1D(Cond6, L=c(0,0,1,0,0,0,0,0,0))
print("Sal slope at rel=3 for older");   contest1D(Cond6, L=c(0,0,1,0,0,0,0,1,0))
print("Rel*sal slope for younger");      contest1D(Cond6, L=c(0,0,0,0,0,1,0,0,0))
print("Rel*sal slopefor older");         contest1D(Cond6, L=c(0,0,0,0,0,1,0,0,1))

############## Make a plot of predicted outcomes ################
# Import fake cases with centered predictor values to be plotted: 
#     rel3=-2to2; sal3=-2to1, older=0or1, yrs65=0or15 
Fake = read_excel(paste0(filesave,filename),sheet="fake") 
# Convert to data frame to use in analysis
Fake = as.data.frame(Fake)
# Merge predicted values from main-effects-only model into FI data
Fake = data.frame(Fake, yhat=predict(object=Cond6, newdata=Fake, re.form=NA))
# Create proper x-axis for plot
Fake$rel = Fake$rel3 + 3

# Make and save plot
png(file = "Example2_Plot.png")  # open file
# Make plot
plot(y=Fake$yhat, x=Fake$rel, type="n", ylim=c(-1,3), xlim=c(1,5),
     xlab="Relevance", ylab="Predicted Log-Transformed RT")  #lty=linetype
lines(x=Fake$rel[1:5],   y=Fake$yhat[1:5],   type="l", lty=1, lwd=2, col="purple1")
lines(x=Fake$rel[6:10],  y=Fake$yhat[6:10],  type="l", lty=1, lwd=2, col="blue1")
lines(x=Fake$rel[11:15], y=Fake$yhat[11:15], type="l", lty=1, lwd=2, col="red1")
lines(x=Fake$rel[16:20], y=Fake$yhat[16:20], type="l", lty=2, lwd=2, col="purple1")
lines(x=Fake$rel[21:25], y=Fake$yhat[21:25], type="l", lty=2, lwd=2, col="blue1")
lines(x=Fake$rel[26:30], y=Fake$yhat[26:30], type="l", lty=2, lwd=2, col="red1")
legend(col=c("purple1","blue1","red1","purple1","blue1","red1"), 
       lty=c(1,1,1,2,2,2), lwd=c(2,2,2,2,2,2),"bottomleft", 
       legend=c("Age=80 Salience=1",  "Age=65 Salience=1",
                "Younger Salience=1", "Age=80 Salience=4",
                "Age=65 Salience=4",  "Younger Salience=4")) 
dev.off()  # close file
##################################################################

print("Print stored variance components table for reference")
as.data.frame(VarCorr(Cond6))
# Save each variance as an object
Cond6SubIntVar  = as.data.frame(VarCorr(Cond6))[1,4]
Cond6SubSalVar  = as.data.frame(VarCorr(Cond6))[2,4]
Cond6ItemIntVar = as.data.frame(VarCorr(Cond6))[4,4]
Cond6ResVar     = as.data.frame(VarCorr(Cond6))[5,4]
# Compute pseudo-R2 for each variance and change relative to Cond3 as new baseline
Cond6SubIntR2        = (Cond3SubIntVar  - Cond6SubIntVar) /Cond3SubIntVar;  Cond6SubIntR2
Cond6SubSalR2        = (Cond3SubSalVar  - Cond6SubSalVar) /Cond3SubSalVar;  Cond6SubSalR2
Cond6ItemIntR2       = (Cond3ItemIntVar - Cond6ItemIntVar)/Cond3ItemIntVar; Cond6ItemIntR2
Cond6ResR2           = (Cond3ResVar     - Cond6ResVar)    /Cond3ResVar;     Cond6ResR2

# Compute totalR2: squared correlation between y and yhat from fixed effects
print("Save yhat and correlation of yhat with y") 
Example2$Yhat = predict(Cond6, re.form=NA)
Cond6r = cor.test(Example2$Yhat, Example2$logRT, method="pearson")
print("Total R2 = "); Cond6r$estimate^2


#################################################################################
# I could not figure out how to get heterogeneous variances in lme (within nlme)
# at the same time as crossed random effects, so I gave up


#################################################################################
# Example of how to export a .csv file for use in Mplus
# Copy data, replace all missing values with -999 for Mplus
Example2_Mplus = subset(Example2, select=c("SubjectID","ItemID","logRT","older","yrs65","rel3","sal3"))
Example2_Mplus[is.na(Example2_Mplus)] <- -999

# Write to .csv file without column names
write.table(x=Example2_Mplus, col.names=FALSE, row.names=FALSE, sep=",",
            file=paste0(filesave,"Example2_Mplus.csv"))

print("Conditional Model 6R: Remove random subject salience slope for Mplus comparison")
Cond6Mplus = lmer(data=Example2, REML=TRUE, formula=  
              logRT~1+rel3+sal3+older+yrs65+ rel3:sal3+
              rel3:older+ sal3:older+ rel3:sal3:older+
              (1|SubjectID)+(1|ItemID))
print("Show results using Sattherwaite DDF") 
summary(Cond6Mplus, ddf="Satterthwaite")
